
/**
 * Feed class
 *
 */

function Feed(feedXML, feedPath) {
	this.feedXML = feedXML;
	this.feedFormat = null;

	this.title = null;
	this.link = null;
	this.description = null;
	this.items = new Array();
	this.lastPubDate = null;

	if(!feedXML) {
		throw "Empty Feed";
	}

	//----- Added by Higmmer : Start ----->
	this.path = feedPath ? feedPath.replace(/&/g, "&amp;").replace(/"/g, "&quot;") : "";
	this.ignoreEnclosures = getIgnoreEnclosures();
	this.adblock = getAdBlockUrlFilter();
	this.titleblock = getAdBlockTitleFilter();
	//<----- Added by Higmmer : End -----

	var rootNodeName = feedXML.documentElement.localName.toLowerCase();
	if(rootNodeName == "feed") {
		this.parseAtom();
	} else if(rootNodeName == "rss" || rootNodeName == "rdf") {
		this.parseRSS();
	} else {
		throw "Feed has invalid root element";
	}
}

Feed.prototype.clone = function() { // Added by Higmmer
	var F = function(){};
	F.prototype = this;
	var newFeed = new F();
	newFeed.items = this.items.slice(0);
	return newFeed;
}

Feed.prototype.isIgnoreEnclosure = function(aURL) { // Added by Higmmer
	if(!aURL || !aURL.match(/^(http|https|ftp):/)) return true; // invalid scheme
	if(!this.ignoreEnclosures) return false; // nothing to ignore

	var ext = "";
	var i = aURL.search(/\.[^.\\/:*?"<>|]+$/);
	if(i >= 0) ext = aURL.substring(i + 1); // get file extention

	var c1 = 0; var c2 = 0;
	for each(var igext in this.ignoreEnclosures) {
		if(igext == "." && ext == "") { // no extention
			return true;
		} else if(igext.charAt(0) != "!") { // positive matching
			if(igext.toLowerCase() == ext.toLowerCase()) return true; // ignore
		} else { // negative matching
			c1++;
			igext = igext.substring(1);
			if(igext == "") return true; // special ignore (for all)
			if(igext.toLowerCase() != ext.toLowerCase()) c2++; // exclude?
		}
	}
	return c1 > 0 ? (c1 == c2) : false;
}

Feed.prototype.checkAdBlock = function(aTitle, aURL) { // Added by Higmmer
	if(this.titleblock && aTitle) {
		var title = aTitle.toLowerCase();
		for each(var str in this.titleblock) {
			if(title.indexOf(str.toLowerCase()) != -1) return true;
		}
	}
	if(this.adblock && aURL) {
		if(this.adblock.test(aURL)) return true;
	}
	return false;
}

Feed.prototype.parseRSS = function() {

	var feedXML = this.feedXML;

	const nsIURIFixup = Components.interfaces.nsIURIFixup;
	const URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(nsIURIFixup);

	var firstElement = feedXML.documentElement;

	if (firstElement.localName.toLowerCase() == "rdf") {
		this.feedFormat = "RSS (1.0)";
	} else if (firstElement.localName.toLowerCase() == "rss") {
		if(firstElement.hasAttribute("version")) {
			this.feedFormat = "RSS (" + firstElement.getAttribute("version") + ")";
		} else {
			this.feedFormat = "RSS (?)";
		}
	}

	var i, j;

	var channelNode;
	for (i = firstElement.firstChild; i != null; i = i.nextSibling) {
		if(i.nodeType != i.ELEMENT_NODE) continue;
		if(i.localName.toLowerCase() == "channel") {
			channelNode = i;
		}
	}
	if (!channelNode) {
		throw "No channel element where expected";
	}

	if (feedXML.getElementsByTagName("channel").length != 0) {
		channelNode = feedXML.getElementsByTagName("channel")[0];
	} else {
		throw "No elements in channel tag";
	}

	for (i = channelNode.firstChild; i != null; i = i.nextSibling) {
		if (i.nodeType != i.ELEMENT_NODE) continue;
		switch(i.nodeName) { // Replaced by Higmmer : localName -> nodeName (from Sage 1.4)
			case "title":
				this.title = entityDecode(CommonFunc.getInnerText(i));
				break;
			case "link":
				//dump("link node: " + i.namespaceURI + "\n"); // Removed by Higmmer
				if (!this.link && i.namespaceURI != "http://www.w3.org/2005/Atom") { // Modified by Higmmer
					this.link = CommonFunc.getInnerText(i);
				}
				break;
			case "description":
				this.description = entityDecode(CommonFunc.getInnerText(i));
				break;
		}
	}

	var itemNodes = feedXML.getElementsByTagName("item");
	var item, guid;
	for(i = 0; itemNodes.length > i; i++) {
		if (itemNodes[i].prefix) continue; // Added by Higmmer : skip elements with namespace (from Sage 1.4)

		item = {title:"", link:"", content:"", pubDate:"", author:"", category:"", enclosure:""}; // Modified by Higmmer
		guid = null;

		for (j = itemNodes[i].firstChild; j!=null; j=j.nextSibling) {
			if (j.nodeType != j.ELEMENT_NODE) continue;
			switch(j.nodeName) { // Replaced by Higmmer : localName -> nodeName (from Sage 1.4)
				case "title":
					item.title = entityDecode(CommonFunc.getInnerText(j));
					break;
				case "link":
					if (!item.link) {
						try { // Modified by Higmmer
							item.link = this.link ? URIFixup.createFixupURI(this.link, nsIURIFixup.FIXUP_FLAG_NONE).resolve(CommonFunc.getInnerText(j)) : CommonFunc.getInnerText(j);
						} catch(e) {
						}
					}
					break;
				case "guid":
					if(!guid) {
						guid = CommonFunc.getInnerText(j);
					}
					break;
				case "description":
					if (!item.content) {
						item.content = CommonFunc.getInnerText(j);
					}
					break;
				case "content:encoded":
					item.content = CommonFunc.getInnerText(j);
					break;
				case "pubDate":
					var tmp_str = CommonFunc.getInnerText(j);
					var tmp_date = rfc822ToJSDate(tmp_str) || iso8601ToJSDate(tmp_str); // Modified by Higmmer
					if(tmp_date) {
						item.pubDate = tmp_date;
					} else {
						logMessage("unable to parse date string: " + tmp_str + "\nfeed: " + this.title); // Modified by Higmmer
					}
					break;
				case "dc:date":
					tmp_str = CommonFunc.getInnerText(j);
					tmp_date = iso8601ToJSDate(tmp_str) || rfc822ToJSDate(tmp_str); // Modified by Higmmer (inspired by Sage-Too)
					if(tmp_date) {
						item.pubDate = tmp_date;
					} else {
						logMessage("unable to parse date string: " + tmp_str + "\nfeed: " + this.title); // Modified by Higmmer
					}
					break;
				//----- Added by Higmmer : Start ----->
				case "dc:creator":
				case "author":
					item.author = entityDecode(CommonFunc.getInnerText(j));
					break;
				case "dc:subject":
				case "category":
					item.category += (item.category ? ", " : "") + entityDecode(CommonFunc.getInnerText(j));
					break;
				case "enclosure":
					if(!item.enclosure && j.hasAttribute("url")) {
						// (inspired by JeffT's modification on Sage-Too)
						var enc = {};
						enc.url = j.getAttribute("url");
						enc.length = j.hasAttribute("length") ? j.getAttribute("length") : -1;
						enc.type = j.hasAttribute("type") ? j.getAttribute("type") : "";
						if(!this.isIgnoreEnclosure(enc.url)) item.enclosure = new ItemEnclosure(enc.url, enc.length, enc.type);
					}
					break;
				//<----- Added by Higmmer : End -----
			}
		}

		if (!item.link && guid) {
			try { // Modified by Higmmer
				item.link = this.link ? URIFixup.createFixupURI(this.link, nsIURIFixup.FIXUP_FLAG_NONE).resolve(guid) : guid;
			} catch(e) {
			}
		}

		if (this.checkAdBlock(item.title, item.link)) { // Added by Higmmer
			logMessage("Ad-Blocked: " + item.title + "\n[" + item.link + "]");
			continue;
		}

		var tmpFeedItem = new FeedItem(item.title, item.link, item.content, item.pubDate, null, item.author, item.category, item.enclosure); // Modified by Higmmer

		if (tmpFeedItem.hasPubDate()) {
			if (tmpFeedItem.getPubDate() > this.lastPubDate) {
				this.lastPubDate = tmpFeedItem.getPubDate();
			}
		}

		this.items.push(tmpFeedItem);
	}
}

Feed.prototype.parseAtom = function() {

	var feedXML = this.feedXML;

	const nsIURIFixup = Components.interfaces.nsIURIFixup;
	const URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(nsIURIFixup);

	var firstElement = feedXML.documentElement;

	if (firstElement.hasAttribute("xmlns") && firstElement.getAttribute("xmlns") == "http://www.w3.org/2005/Atom") {
		this.feedFormat = "Atom (1.0)";
	} else if (firstElement.hasAttribute("version")) {
		this.feedFormat = "Atom (" + firstElement.getAttribute("version") + ")";
	} else {
		this.feedFormat = "Atom (?)";
	}

	// xml:base support for <feed> element
	var baseURI;
	if (firstElement.hasAttribute("xml:base")) {
		baseURI = firstElement.getAttribute("xml:base");
	} else { // Added by Higmmer : substitute feed path for base url if xml:base isn't specified
		try {
			baseURI = URIFixup.createFixupURI(this.path, nsIURIFixup.FIXUP_FLAG_NONE).resolve(".");
			logMessage("xml:base isn't specified in Atom feed\nsubstituted by: " + baseURI);
		} catch(e) {
		}
	}

	var i, j, z;

	for (var i = feedXML.documentElement.firstChild; i != null; i = i.nextSibling) {
		if (i.nodeType != i.ELEMENT_NODE) continue;
		if (i.namespaceURI.search(/atom/i) == -1) continue; // Added by Higmmer : skip non-Atom elements (from Sage 1.4)
		switch(i.localName) {
			case "title":
				this.title = entityDecode(CommonFunc.getInnerText(i));
				break;
			case "link":
				if (!this.link && (i.hasAttribute("rel") && i.getAttribute("rel").toLowerCase() == "alternate") || !i.hasAttribute("rel")) { // Modified by Higmmer
					if (baseURI) {
						try { // Modified by Higmmer
							this.link = URIFixup.createFixupURI(baseURI, nsIURIFixup.FIXUP_FLAG_NONE).resolve(i.getAttribute("href"));
						} catch(e) {
						}
					} else {
						this.link = i.getAttribute("href");
					}
				}
				break;
			case "subtitle":
			case "tagline":
				this.description = entityDecode(CommonFunc.getInnerText(i));
				break;
		}
	}

	var entryNodes = feedXML.getElementsByTagName("entry");
	for (i = 0; entryNodes.length > i; i++) {
		var item = {title:"", link:"", content:"", pubDate:"", baseURI:"", author:"", category:"", enclosure:""}; // Modified by Higmmer

		// xml:base support for <entry> element
		if (entryNodes[i].hasAttribute("xml:base")) {
			if (baseURI) {
				try { // Modified by Higmmer
					item.baseURI = URIFixup.createFixupURI(baseURI, nsIURIFixup.FIXUP_FLAG_NONE).resolve(entryNodes[i].getAttribute("xml:base"));
				} catch(e) {
				}
			} else {
				item.baseURI = entryNodes[i].getAttribute("xml:base");
			}
		} else { // Added by Higmmer : substitute feed's base url for items' if xml:base isn't specified
			item.baseURI = baseURI;
		}

		var titleNodes = entryNodes[i].getElementsByTagName("title");
		if (titleNodes.length) {
			item.title = entityDecode(CommonFunc.getInnerText(titleNodes[0]));
		}

		var linkNodes = entryNodes[i].getElementsByTagName("link");
		if (linkNodes.length) {
			for (j = 0; j < linkNodes.length; j++) {
				if(!item.link) { // Modified by Higmmer
					if (!linkNodes[j].hasAttribute("rel") || linkNodes[j].getAttribute("rel").toLowerCase() == "alternate") {
						try {
							item.link = item.baseURI ?
							            URIFixup.createFixupURI(item.baseURI, nsIURIFixup.FIXUP_FLAG_NONE).resolve(linkNodes[j].getAttribute("href"))
							          : linkNodes[j].getAttribute("href");
						} catch(e) {
						}
					}
				} else if(!item.enclosure) { // Added by Higmmer
					if(linkNodes[j].hasAttribute("rel") && linkNodes[j].getAttribute("rel").toLowerCase() == "enclosure") {
						try {
							// (inspired by JeffT's modification on Sage-Too)
							var enc = {};
							enc.url = item.baseURI ?
							    URIFixup.createFixupURI(item.baseURI, nsIURIFixup.FIXUP_FLAG_NONE).resolve(linkNodes[j].getAttribute("href"))
							  : linkNodes[j].getAttribute("href");
							enc.length = linkNodes[j].hasAttribute("length") ? linkNodes[j].getAttribute("length") : -1;
							enc.type = linkNodes[j].hasAttribute("type") ? linkNodes[j].getAttribute("type") : "";
							if(!this.isIgnoreEnclosure(enc.url)) item.enclosure = new ItemEnclosure(enc.url, enc.length, enc.type);
						} catch(e) {
						}
					}
				}
			}
		}

		//----- Added by Higmmer (from Sage 1.4) : Start ----->
		var authorNodes = entryNodes[i].getElementsByTagName("author");
		if (authorNodes.length > 0) {
			var nameNodes = authorNodes[0].getElementsByTagName("name");
			if (nameNodes.length > 0) authorNodes = nameNodes;
			item.author = entityDecode(CommonFunc.getInnerText(authorNodes[0]));
		}
		//<----- Added by Higmmer : End -----

		//----- Added by Higmmer : Start ----->
		var categoryNodes = entryNodes[i].getElementsByTagName("category");
		if(categoryNodes.length == 0) categoryNodes = entryNodes[i].getElementsByTagName("subject");
		for(var j = 0; j < categoryNodes.length; j++) {
			var category = "";
			switch(categoryNodes[j].nodeName) {
			case "category":
				category = categoryNodes[j].getAttribute(categoryNodes[j].hasAttribute("label") ? "label" : "term");
				break;
			case "dc:subject":
				category = CommonFunc.getInnerText(categoryNodes[j]);
				break;
			}
			if(category) {
				item.category += (item.category ? ", " : "") + entityDecode(category);
			}
		}
		//<----- Added by Higmmer : End -----

		var updatedNodes = entryNodes[i].getElementsByTagName("updated");
		if (updatedNodes.length) {
			var tmp_str = CommonFunc.getInnerText(updatedNodes[0]);
			var tmp_date = iso8601ToJSDate(tmp_str) || rfc822ToJSDate(tmp_str); // Modified by Higmmer
			if (tmp_date) {
				item.pubDate = tmp_date;
			} else {
				logMessage("unable to parse date string: " + tmp_str + "\nfeed: " + this.title); // Modified by Higmmer
			}
		}

		var issuedNodes = entryNodes[i].getElementsByTagName("issued");
		if (issuedNodes.length) {
			tmp_str = CommonFunc.getInnerText(issuedNodes[0]);
			tmp_date = iso8601ToJSDate(tmp_str) || rfc822ToJSDate(tmp_str); // Modified by Higmmer
			if (tmp_date) {
				item.pubDate = tmp_date;
			} else {
				logMessage("unable to parse date string: " + tmp_str + "\nfeed: " + this.title); // Modified by Higmmer
			}
		}

		var aEntryNode = entryNodes[i];

		var contentNodes = aEntryNode.getElementsByTagName("content");
		var contentHash = {};
		var contentString;
		var xmlSerializer = new XMLSerializer();
		for (j = 0; j < contentNodes.length; j++) {
			var contType = contentNodes[j].getAttribute("type");
			if (contType == "application/xhtml+xml" || contType == "xhtml") {
				contentString = "";
				for(z = 0; z < contentNodes[j].childNodes.length; z++) {
					contentString += xmlSerializer.serializeToString(contentNodes[j].childNodes[z]);
				}
			} else {
				contentString = CommonFunc.getInnerText(contentNodes[j]);
			}
			contentHash[contType] = contentString;
		}

		var summaryNodes = aEntryNode.getElementsByTagName("summary");

		if ("application/xhtml+xml" in contentHash) {
			item.content = contentHash["application/xhtml+xml"];
		} else if ("xhtml" in contentHash) {
			item.content = contentHash["xhtml"];
		} else if ("text/html" in contentHash) {
			item.content = contentHash["text/html"];
		} else if ("html" in contentHash) {
			item.content = contentHash["html"];
		} else if ("text/plain" in contentHash) {
			item.content = contentHash["text/plain"];
		} else if ("text" in contentHash) {
			item.content = contentHash["text"];	
		} else if (summaryNodes.length) {
			item.content = CommonFunc.getInnerText(summaryNodes[0]);
		}

		if (this.checkAdBlock(item.title, item.link)) { // Added by Higmmer
			logMessage("Ad-Blocked: " + item.title + "\n[" + item.link + "]");
			continue;
		}

		var tmpFeedItem = new FeedItem(item.title, item.link, item.content, item.pubDate, item.baseURI, item.author, item.category, item.enclosure); // Modified by Higmmer

		if (tmpFeedItem.hasPubDate()) {
			if (tmpFeedItem.getPubDate() > this.lastPubDate) {
				this.lastPubDate = tmpFeedItem.getPubDate();
			}
		}

		this.items.push(tmpFeedItem);
	}
}

Feed.prototype.getPath = function() { // Added by Higmmer
	return this.path;
}

Feed.prototype.getTitle = function() {
	return this.title ? htmlToText(this.title) : "(No Title)"; // Modified by Higmmer
}

Feed.prototype.getDecodedTitle = function() { // Added by Higmmer
	return entityDecode(this.getTitle());
}

Feed.prototype.hasDescription = function() {
	if(!this.description) {
		return false;
	} else {
		return true;
	}
}

Feed.prototype.getDescription = function() {
	if(this.hasDescription()) {
		return htmlToText(this.description); // Modified by Higmmer
	} else {
		return "";
	}
}

Feed.prototype.getLink = function() {
	const URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(Components.interfaces.nsIURIFixup);
	var urlString;
	try {
		var uri = URIFixup.createFixupURI(this.link, URIFixup.FIXUP_FLAG_NONE);
		if (uri.scheme.match(/^(http|https)$/)) { // Modified by Higmmer
			urlString = uri.spec;
		} else {
			urlString = "";
		}
	} catch (e) {
		urlString = "";
	}
	return urlString ? urlString : "about:blank"; // Modified by Higmmer
}

Feed.prototype.hasLastPubDate = function() {
	if(!this.lastPubDate) {
		return false;
	} else {
		return true;
	}
}

Feed.prototype.getLastPubDate = function() {
	if(this.hasLastPubDate()) {
		return this.lastPubDate;
	} else {
		return null;
	}
}

Feed.prototype.getItemCount = function() {
	return this.items.length;
}

Feed.prototype.getItem = function(itemIndex) {
	return this.items[itemIndex];
}

//----- Modified by Higmmer : Start ----->
Feed.prototype.getItems = function(sort) {
	if(sort.lastIndexOf("chrono", 0) == 0 && !this.hasLastPubDate()) {  // if the feed doesn't have pub dates, we're going to do a source sort
		sort = "source";
	}
	var items_array = this.items.slice(0);
	switch(sort) {
		case "chrono":
			function chronoSort(a, b) {
				var a_ts = a.hasPubDate() ? a.getPubDate() : 0;
				var b_ts = b.hasPubDate() ? b.getPubDate() : 0;
				return b_ts - a_ts;
			}
			items_array.sort(chronoSort);
			break;
		case "chrono_inv":
			function chronoSortInv(a, b) {
				var a_ts = a.hasPubDate() ? a.getPubDate() : Infinity;
				var b_ts = b.hasPubDate() ? b.getPubDate() : Infinity;
				return a_ts - b_ts;
			}
			items_array.sort(chronoSortInv);
			break;
		/*case "source":
			items_array = this.items;
			break;
		default:
			items_array = this.items;
			break;*/
	}
	return items_array;
}
//<----- Modified by Higmmer : End -----

Feed.prototype.getFormat = function() {
	return this.feedFormat;
}

Feed.prototype.getSignature = function() {
	var hashText = "";
	for(var c = 0; c < this.getItemCount(); c++) {
		hashText += this.getItem(c).getTitle();
	}
	var sig = "[" + b64_sha1(hashText) + "]";
	return sig;
}



/**
 * FeedItem class
 *
 */

function FeedItem(title, link, content, pubDate, baseURI, author, category, enclosure) { // Modified by Higmmer
	this.title = title;
	this.link = link;
	this.content = content;
	this.pubDate = pubDate;
	this.baseURI = baseURI;
	this.author = author;       // Added by Higmmer
	this.category = category;   // Added by Higmmer
	this.enclosure = enclosure; // Added by Higmmer
}

FeedItem.prototype.hasTitle = function() {
	if(!this.title) {
		return false;
	} else {
		return true;
	}
}

FeedItem.prototype.getTitle = function() {
	var title;
	if (this.hasTitle()) {
		title = htmlToText(this.title); // Modified by Higmmer
	} else {
		if (this.hasContent()) {
			var temp = htmlToText(this.getContent()); // Modified by Higmmer
			temp = temp.replace(/\n|\t/g,' ').replace(/\s+/g, ' '); // Modified by Higmmer
			title = temp.substring(0, 30) + "...";
		} else {
			title = ""; // Modified by Higmmer
		}
	}
	return title ? title : "(No Title)"; // Modified by Higmmer
}

FeedItem.prototype.getDecodedTitle = function() { // Added by Higmmer
	return entityDecode(this.getTitle());
}

FeedItem.prototype.getLink = function() {
	const URIFixup = Components.classes["@mozilla.org/docshell/urifixup;1"].getService(Components.interfaces.nsIURIFixup);
	var urlString;
	try {
		var uri = URIFixup.createFixupURI(this.link, URIFixup.FIXUP_FLAG_NONE);
		if (uri.scheme.match(/^(http|https|ftp)$/)) { // Modified by Higmmer
			urlString = uri.spec;
		} else {
			urlString = "";
		}
	} catch (e) {
		urlString = "";
	}
	return urlString ? urlString : "about:blank"; // Modified by Higmmer
}

FeedItem.prototype.hasContent = function() {
	if(!this.content) {
		return false;
	} else {
		return true;
	}
}

FeedItem.prototype.getContent = function() {
	if(this.hasContent()) {
		return this.content;
	} else {
		return "No content";
	}
}

FeedItem.prototype.hasPubDate = function() {
	if(!this.pubDate) {
		return false;
	} else {
		return true;
	}
}

FeedItem.prototype.getPubDate = function() {
	if(this.hasPubDate()) {
		return this.pubDate;
	} else {
		return null;
	}
}

FeedItem.prototype.hasBaseURI = function() {
	return Boolean(this.baseURI);
}

FeedItem.prototype.getBaseURI = function() {
	return this.hasBaseURI() ? this.baseURI : null;
}

//----- Added by Higmmer : Start ----->
FeedItem.prototype.hasAuthor = function() {
	return Boolean(this.author);
}

FeedItem.prototype.getAuthor = function() {
	return this.hasAuthor() ? htmlToText(this.author) : null;
}

FeedItem.prototype.hasCategory = function() {
	return Boolean(this.category);
}

FeedItem.prototype.getCategory = function() {
	return this.hasCategory() ? htmlToText(this.category) : null;
}

FeedItem.prototype.hasEnclosure = function() {
	return Boolean(this.enclosure);
}

FeedItem.prototype.getEnclosure = function() {
	return this.hasEnclosure() ? this.enclosure : null;
}
//<----- Added by Higmmer : End -----

//----- Added by JeffT : Start ----->
function ItemEnclosure(url, length, type) {
	this.url = url;
	this.length = Number(length); // Modified by Higmmer
	this.type = type;
}

ItemEnclosure.prototype.getURL = function() {
	return this.url;
}

ItemEnclosure.prototype.getLength = function() {
	return this.length;
}

ItemEnclosure.prototype.getType = function() {
	return this.type;
}
//<----- Added by JeffT : End -----


/**
 * Utility functions
 *
 */
 
// Parses an RFC 822 formatted date string and returns a JavaScript Date object, returns null on parse error
// Example inputs:  "Sun, 08 May 05 15:19:37 GMT"  "Mon, 09 May 2005 00:50:19 GMT"
function rfc822ToJSDate(date_str) {
	var date_array = date_str.split(" ");
	// check for two digit year
	if(date_array.length == 6 && date_array[3].length == 2) {
		// convert to four digit year with a pivot of 70
		if(date_array[3] < 70) {
			date_array[3] = "20" + date_array[3];
		} else {
			date_array[3] = "19" + date_array[3];
		}
	}
	//----- Added by Higmmer : Start ----->
	if(date_array.length == 6) {
		date_array[5] = date_array[5].toUpperCase();
		switch(date_array[5]) {
		case "Z":   date_array[5] = "UT"; break;
		case "JST": date_array[5] = "+0900"; break;
		}
	}
	//<----- Added by Higmmer : End -----
	date_str = date_array.join(" ");
	var date = new Date(date_str);
	if(date != "Invalid Date") {
		return date;
	} else {
		return null
	}
}

// Parses an ISO 8601 formatted date string and returns a JavaScript Date object, returns null on parse error
// Example inputs:  "2004-06-17T18:00Z" "2004-06-17T18:34:12+02:00"
function iso8601ToJSDate(date_str) {
	var tmp = date_str.toUpperCase().replace(" ", "T").split("T"); // Modified by Higmmer (fix separator)
	var date = tmp[0];

	date = date.split("-");
	var year = date[0];
	var month = date[1];
	var day = date[2];

	var hours = 0;
	var minutes = 0;
	var seconds = 0;
	var tz_mark = "Z";
	var tz_hours = 0;
	var tz_minutes = 0;
	var time, whole_time, tz;

	if(tmp.length == 2) {
		if(/^[Z+-]/.test(tmp[1])) { // Added by Higmmer (complement time if not specified)
			tmp[1] = "00:00" + tmp[1];
		}
		whole_time = tmp[1];
		tz_mark = whole_time.match("[Z+-]{1}");
		if(!tz_mark) { // Added by Higmmer (assume local time-zone if not specified)
			var timezone = (new Date()).toTimeString().split("GMT")[1] || "+0000";
			tz_mark = timezone.charAt(0);
			whole_time += timezone;
		}
		if(tz_mark) {
			tmp = whole_time.split(tz_mark);
			time = tmp[0];
			if(tz_mark == "+" || tz_mark == "-") {
				tz = tmp[1];
				if(tz.indexOf(":") == -1) { // Added by Higmmer (ISO8601 allows no-colon time-zone)
					if(tz.match(/^([0-9]{2})([0-9]{2})?$/)) tz = RegExp.$1 + ":" + (RegExp.$2 || "00");
				}
				tmp = tz.split(":");
				tz_hours = tmp[0];
				tz_minutes = tmp[1];
			}
		} else { // These block is obsolete (by Higmmer)
			tz_mark = "Z";
			time = whole_time;
		}
		tmp = time.split(":");
		hours = tmp[0];
		minutes = tmp[1];
		if(tmp.length == 3) {
			seconds = tmp[2];
		}
	}

	var utc = Date.UTC(year, month - 1, day, hours, minutes, seconds);
	var tmp_date;
	if(tz_mark == "Z") {
		tmp_date = new Date(utc);
	} else if(tz_mark == "+") {
		tmp_date = new Date(utc - tz_hours*3600000 - tz_minutes*60000);
	} else if(tz_mark == "-") {
		tmp_date = new Date(utc + tz_hours*3600000 + tz_minutes*60000);
	} else {
		tmp_date = "Invalid Date";
	}

	if (tmp_date == "Invalid Date") {
		return null;
	} else {
		return tmp_date;
	}
}

function entityDecode(aStr) {
	var	formatConverter = Components.classes["@mozilla.org/widget/htmlformatconverter;1"].createInstance(Components.interfaces.nsIFormatConverter);
	var fromStr = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
	fromStr.data = aStr;
	var toStr = { value: null };

	try {
		formatConverter.convert("text/html", fromStr, fromStr.toString().length, "text/unicode", toStr, {});
	} catch(e) {
		return aStr;
	}
	if(toStr.value) {
		toStr = toStr.value.QueryInterface(Components.interfaces.nsISupportsString);
		return toStr.toString();
	}
	return aStr;
}


//----- Added by Higmmer : Start ----->
function getIgnoreEnclosures() {
	var ignoreEnclosures = null;
	try {
		var tmp = CommonFunc.getPrefValue(CommonFunc.ENCLOSURE_IGNORE_FILETYPES, "str", "jpg jpeg png gif exe");
		if(tmp.length > 0) {
			ignoreEnclosures = tmp.replace(/^\s+|\s+$|\s(?=\s+)/g, "").split(" ");
			if(ignoreEnclosures[0] == "") ignoreEnclosures = null;
		}
	} catch(e) {
	}
	return ignoreEnclosures;
}

function getAdBlockUrlFilter() {
	var adblock = null;
	try {
		var tmp = CommonFunc.getPrefValue(CommonFunc.FILTER_ADBLOCK_REGEXP, "str", "");
		if(tmp) adblock = new RegExp(tmp, "i");
	} catch(e) {
	}
	return adblock;
}

function getAdBlockTitleFilter() {
	var titleblock = null;
	try {
		var tmp = CommonFunc.getPrefValue(CommonFunc.FILTER_ADBLOCK_TITLE, "wstr", "AD: PR: [PR]");
		if(tmp.length > 0) {
			titleblock = tmp.replace(/^\s+|\s+$|\s(?=\s+)/g, "").split(" ");
			if(titleblock[0] == "") titleblock = null;
		}
	} catch(e) {
	}
	return titleblock;
}
//<----- Added by Higmmer : End -----
